# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Phoenix87 <phoenix1987@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gtumbler_lib.preferences import preferences

import shelve
import tempfile
import os
import collections

class TempContext:
	def __init__(self):
		self.tmp_files = []
		
	def __del__(self):
		self.clear()
	
	def get_files(self):
		return self.tmp_files
	
	def new_file(self, ext = ''):
		fd, tmp = tempfile.mkstemp(ext)
		os.close(fd)
		self.tmp_files.append(tmp)
		return tmp
		
	def clear(self):
		for f in self.tmp_files:
			try:
				os.remove(f)
			except OSError:
				pass

## Singleton implementation of a registry class
#

UNDO_MAX_LEN = 20

class Session:
	__shared_state = {
		'version' 			: '0.1',
		'doc_uid' 			: 0,
		'doc_crop'			: {},
		'doc_temp'			: {},
		'doc_ranges'		: {},
		'boxes'				: [[0] * 4 for j in range(5)],
		'history'			: collections.deque(maxlen = UNDO_MAX_LEN),
		'future'			: [],
		'filename'			: None,
		'has_changed'		: 0,
		'filename'			: None,
		'temp'				: {}
	}
	
	def __init__(self, main_win = None):
		self.__dict__ = self.__shared_state
		if main_win: self.main = main_win

	def _can(self, what):
		action = {'redo' : [self.future, ['tb_redo', 'mnu_edit_redo']],
		          'undo' : [self.history, ['tb_undo', 'mnu_edit_undo']]}
		p, widgets = action[what]
		can = bool(p)
		for widget in widgets:
			getattr(self.main.ui, widget).set_sensitive(can)
		return can

	def _dump_doc_list(self):
		r = []
		it = self.main.ui.ls_documents.get_iter_first()
		while it:
			row = []
			for i in range(self.main.ui.ls_documents.get_n_columns()):
				row.append(self.main.ui.ls_documents.get_value(it, i))
			r.append(row)
			it = self.main.ui.ls_documents.iter_next(it)
		return r

	def _undump_doc_list(self, src):
		self.main.ui.ls_documents.clear()
		for row in src:
			self.main.ui.ls_documents.append(row)

	def _update_window_title(self):
		try:
			path, fn = os.path.split(self.filename)
		except AttributeError:
			fn = "Unsaved session"
			path = ""
		self.main.set_title("%s%s%s - gtumbler" % ("*" if bool(self.has_changed) else "", \
		                                            fn if self.filename else 'Unsaved session', \
		                                            " (%s)" % path if path and path != "" and preferences['chk_display_path'] else ""))

	def assign_uid(self):
		self.doc_uid += 1
		return self.doc_uid - 1
	
	def can_redo(self):
		return self._can('redo')
	
	def can_undo(self):
		return self._can('undo')
	
	def clear(self):
		self.doc_uid = 0
		self.history = collections.deque(maxlen=20),
		self.has_changed += 1

	def do(self, cmd):
		cmd.do()
		self.history.append(cmd)
		self.future = []
		self.has_changed += 1
		self._update_window_title()
		
		self.can_redo()
		return self.can_undo()

	def new(self):
		self.main.ui.ls_documents.clear()
		self.doc_uid = 0
		self.doc_temp = {}
		self.doc_ranges = {}
		self.filename = None
		
		self.history = collections.deque(maxlen = UNDO_MAX_LEN)
		self.future = []
		self.can_redo()
		self.can_undo()
		
		self.has_changed = False
		self._update_window_title()
		
	def open(self, filename): ## TODO: handle gtumbler versions
		shelf = shelve.open(filename)
		try:
			self._undump_doc_list(shelf['ls_doc'])
			self.doc_uid = shelf['doc_uid']
			self.doc_ranges = shelf['doc_ranges']
			
			self.filename = filename
			self.has_changed = 0
			self._update_window_title()
			
		except (KeyError, ValueError):
			return False

		finally: ## WARNING: The logic of this is not so convincing...
			shelf.close()
			self.status('File "%s" has been loaded successfully.' % self.filename)

		return True

	def redo(self):
		if not self.can_redo(): return False
		
		cmd = self.future.pop()
		cmd.redo()
		self.history.append(cmd)

		self.has_changed += 1
		self._update_window_title()
		
		self.can_undo()
		return self.can_redo()

	def save(self, fn = None):
		if fn:
			filename = fn
			self.has_changed = True
		else: filename = self.filename
		if not filename: return False
		if not self.has_changed:
			self.status("No changes need to be saved.")
			return False

		try:
			shelf = shelve.open(filename, 'n')
			shelf['ls_doc'] = self._dump_doc_list()
			shelf['doc_uid'] = self.doc_uid
			shelf['doc_ranges'] = self.doc_ranges
			shelf['version'] = self.version
			shelf.close()
			self.filename = filename
			self.status('File "%s" has been saved successfully.' % self.filename)

		except IOError:
			self.status('Unable to save "%s".' % self.filename)
			return False
			
		self.has_changed = 0
		self._update_window_title()
		return True
	
	def status(self, event):
		self.main.ui.sb_main.push(0, event)
	
	def temp_new(self, name):
		# Return the existing context if already present
		if name in self.temp.keys(): return self.temp[name]
		
		# Otherwise create a new one
		self.temp[name] = TempContext()
		return self.temp[name]
		
	def temp_delete(self, name):
		del self.temp[name]

	def undo(self):
		if not self.can_undo(): return False
		
		cmd = self.history.pop()
		cmd.undo()
		self.future.append(cmd)
		
		self.has_changed -= 1
		self._update_window_title()
		
		self.can_redo()
		return self.can_undo()
